"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const prompts_1 = __importDefault(require("prompts"));
const globalConfig_1 = __importDefault(require("./globalConfig"));
const config_1 = require("./config");
const ua_1 = __importDefault(require("./ua"));
const portfinder_1 = require("portfinder");
const http_1 = require("http");
const http_terminator_1 = require("http-terminator");
const opener_1 = __importDefault(require("opener"));
const cliLoginUrl = (port) => `${globalConfig_1.default.WEB_URL}/cli-login?port=${port}`;
const openCliLoginPage = (port) => {
    return new Promise((resolve) => {
        opener_1.default(cliLoginUrl(port), resolve);
    });
};
const cliRedirectUrl = (success) => `${globalConfig_1.default.WEB_URL}/cli-login-${success ? 'success' : 'error'}`;
const startServer = (port) => {
    let httpTerminator = undefined;
    let resolveToken = undefined;
    const tokenPromise = new Promise((resolve) => {
        resolveToken = resolve;
    });
    const server = http_1.createServer((req, res) => {
        const url = new URL('http://localhost' + req.url);
        if (req.method !== 'GET' || url.pathname !== '/') {
            res.statusCode = 400;
            res.end();
            return;
        }
        const token = url.searchParams.get('token');
        const tokenProvided = typeof token === 'string';
        res.setHeader('Location', cliRedirectUrl(tokenProvided));
        res.statusCode = 302;
        res.end();
        httpTerminator === null || httpTerminator === void 0 ? void 0 : httpTerminator.terminate();
        if (tokenProvided) {
            resolveToken(token);
        }
    });
    httpTerminator = http_terminator_1.createHttpTerminator({ server });
    return new Promise((resolve) => {
        server.listen(port, () => resolve({ tokenPromise }));
    });
};
function authorization() {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const config = yield config_1.getConfig();
        if (config === null || config === void 0 ? void 0 : config.authToken) {
            return config;
        }
        else {
            const result = yield prompts_1.default({
                type: 'text',
                name: 'response',
                message: "You don't appear to be logged in. Press any key to open up the browser to login or 'q' to exit:",
            });
            if (result.response === 'q') {
                throw new Error('Exiting.');
            }
            try {
                const port = yield portfinder_1.getPortPromise();
                const { tokenPromise } = yield startServer(port);
                console.log('Waiting for login in the browser...');
                yield openCliLoginPage(port);
                const conf = { authToken: yield tokenPromise };
                console.log('Successfully logged in.');
                try {
                    yield fs_1.promises.writeFile(config_1.configPath, JSON.stringify(conf, null, 2));
                    ua_1.default.event({
                        ec: 'user',
                        ea: 'login',
                        el: 'CLI',
                    }, () => conf);
                }
                catch (e) {
                    return e;
                }
            }
            catch (error) {
                let errorMessage = (_a = error.response.data.error) === null || _a === void 0 ? void 0 : _a.code;
                if (errorMessage === 'EMAIL_PASSWORD_INCORRECT') {
                    errorMessage = 'Sign in failed. Please check your email or password.';
                }
                throw Error(errorMessage);
            }
        }
    });
}
exports.default = authorization;
