#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const fs_2 = require("fs");
const yargs_1 = require("yargs");
const path_1 = __importDefault(require("path"));
const simple_git_1 = __importDefault(require("simple-git"));
const auth_1 = __importDefault(require("./auth"));
const transmit_1 = __importDefault(require("./transmit"));
const config_1 = require("./config");
const run = () => __awaiter(void 0, void 0, void 0, function* () {
    if (yargs_1.argv.h || yargs_1.argv.help) {
        const help = yield fs_2.promises.readFile(path_1.default.resolve(__dirname, '../help.txt'), 'utf-8');
        console.log(help);
    }
    else if (yargs_1.argv.v || yargs_1.argv.version) {
        const npmPackage = JSON.parse(yield fs_2.promises.readFile(path_1.default.resolve(__dirname, '../package.json'), 'utf-8'));
        console.log(npmPackage.version);
    }
    else {
        for (const arg in yargs_1.argv) {
            if (arg) {
                switch (arg) {
                    case '$0':
                    case 'v':
                    case 'version':
                    case 'expires':
                    case 'e':
                    case 'signout':
                    case 'h':
                    case 'help':
                    case '_':
                        break;
                    default:
                        console.error("The option: '" +
                            arg +
                            "' is not accepted. Please see help by running diffchecker --help.");
                        process.exit();
                }
            }
        }
        if (yargs_1.argv.signout) {
            fs_1.default.unlink(config_1.configPath, () => {
                console.log("You've been signed out. Run the command again to sign in.");
                process.exit();
            });
        }
        let fileOneExists = true;
        let fileTwoExists = true;
        try {
            yield auth_1.default();
            const fileOne = yargs_1.argv._[0];
            const fileTwo = yargs_1.argv._[1];
            try {
                yield fs_2.promises.access(fileOne);
            }
            catch (error) {
                fileOneExists = false;
            }
            try {
                yield fs_2.promises.access(fileTwo);
            }
            catch (error) {
                fileTwoExists = false;
            }
            /* Check if there's just one argument. If there is, assume user wants to compare with most recent git commit. Otherwise, read the next argument as a file to compare with. */
            if (yargs_1.argv._.length === 1 && fileOneExists) {
                const options = {
                    baseDir: process.cwd(),
                    binary: 'git',
                    maxConcurrentProcesses: 6,
                };
                try {
                    const git = simple_git_1.default(options);
                    const gitShow = yield git.show([`HEAD:${fileOne}`]);
                    transmit_1.default({
                        left: gitShow,
                        right: yield fs_2.promises.readFile(fileOne, 'utf-8'),
                    });
                }
                catch (error) {
                    console.error("Tried to look for a git version of that file, but couldn't locate a git repository nearby.");
                }
            }
            else if (!fileOneExists) {
                console.error('File(s) not found. Please check your file paths.');
            }
            else if (fileOneExists && fileTwoExists) {
                transmit_1.default({
                    left: yield fs_2.promises.readFile(fileOne, 'utf-8'),
                    right: yield fs_2.promises.readFile(fileTwo, 'utf-8'),
                });
            }
            else {
                console.error(fileOneExists && fileTwoExists
                    ? 'Something went wrong with your command.'
                    : 'File(s) not found. Please check your file paths.');
            }
        }
        catch (error) {
            console.error(error.message);
        }
    }
});
run();
