import opener from 'opener';
import { argv } from 'yargs';
import globalConfig from './globalConfig';
import axios from 'axios';

import { getConfig } from './config';
import ua from './ua';
import checkLanguage from './language-detection';

export default async function transmit({
  left,
  right,
}: {
  left: string;
  right: string;
}): Promise<void> {
  try {
    const config = await getConfig();
    const headers = {
      Authorization: 'Bearer ' + config.authToken,
    };
    const axiosResponse = await axios.post(
      globalConfig.API_URL + '/diffs',
      {
        left,
        right,
        expiry: argv.expires || argv.e || 'forever',
        title: argv.title || null,
      },
      { headers: headers },
    );
    const url = `${globalConfig.WEB_URL}/${await axiosResponse.data.slug}`;
    const language = checkLanguage(left);

    ua.event({
      ec: 'Diffs',
      ea: 'Written in',
      el: language,
    });
    ua.event(
      {
        ec: 'Diffs',
        ea: 'Submitted from',
        el: 'CLI',
      },
      function diffCreateError() {
        console.log('Your diff is ready: ' + url);
        opener(url, () => {
          process.exit();
        });
      },
    );
  } catch (e) {
    console.error(e.message);
    process.exit(1);
  }
}
